@@ -60,10 +60,12 @@ sub collect_current {
     my $self = shift;
     my $config = shift;
     croak "Must pass in a Bryar::Config object" unless UNIVERSAL::isa($config, "Bryar::Config");
-    return sort {$b->epoch <=> $a->epoch }
-                $config->source->search(
-                    $config,
-                    limit => $config->recent());
+
+    my @list = sort { $b->epoch <=> $a->epoch } $config->source->search(
+        $config,
+        limit => $config->recent()
+    );
+    return @list;
 }
 
 
