@@ -7,6 +7,8 @@ our $VERSION = '1.2';
 use Time::Piece;
 use Time::Local;
 use Digest::MD5 qw(md5_hex);
+use Encode;
+use HTTP::Date;
 
 =head1 NAME
 
@@ -84,13 +86,17 @@ sub parse_path {
     #...
 
     my %args;
-    if (defined $pi[-1] and $pi[-1] eq "xml")     { $args{format} = "xml"; pop @pi; }
-    if (defined $pi[-1] and $pi[-1] =~ /id_(.*)/) { $args{id} = $1; pop @pi; }
-    if (defined $pi[0] and $pi[0] =~ /^([a-zA-Z]\w*)/) { # We have a subblog
+    if ($pi[-1] and $pi[-1] eq "xml") { $args{format} = "xml"; pop @pi; }
+    if ($pi[-1] and $pi[-1] =~ /^id_([0-9]+)/) { $args{id} = $1; pop @pi; }
+    if ($pi[0] and $pi[0] =~ /^([a-zA-Z]\w*)/
+               and $pi[0] !~ /^(?:before)_[0-9]+$/) { # We have a subblog
         $args{subblog} = $1;
         shift @pi;
     }
-    if (@pi) { # Time/date handling
+    if (@pi == 1 and $pi[0] =~ /^before_([0-9]+)$/) {
+        $args{before} = $1;
+        $args{limit} = $config->{recent};
+    } elsif (@pi) { # Time/date handling
         my ($from, $til) = _make_from_til(@pi);
         if ($from and $til) {
             $args{before} = $til;
@@ -148,41 +154,93 @@ Output the entire blog data to the browser
 =cut
 
 sub output {
-    my ($self, $ct, $data) = @_;
-    $self->send_header("Content-type", $ct);
-    # $self->send_header('Cache-Control', 'max-age=180');
-    if ($self->_etag($data)) {
+    my ($self, $ct, $data, $last_modified, $headers) = @_;
+    $headers ||= { };
+
+    $self->send_header('Content-Type', $ct) if not $headers->{'Content-Type'};
+
+    if (not $headers->{Status} and $self->not_modified($last_modified, $data)) {
+        $self->send_header($_, $headers->{$_}) foreach keys %$headers;
         $self->send_header('Status', '304 Not Modified');
         $self->send_header('Content-Length', 0);
         $self->send_data('');
     } else {
-        $self->send_header('Content-Length', length($data));
+        $self->send_header($_, $headers->{$_}) foreach keys %$headers;
+        $self->send_header('Content-Length', bytes::length($data));
         $self->send_data($data);
     }
 }
 
-sub _etag {
+sub not_modified {
+    my ($self, $last_modified, $data) = @_;
+
+	# Each method outputs a header as a side effect, so they cannot be
+	# combined in a single test.
+	my $t1 = $self->check_last_modified($last_modified);
+	my $t2 = $self->check_etag($data);
+	return $t1 and $t2;
+}
+
+sub check_etag {
     my ($self, $data) = @_;
     my $req_tag = $self->get_header("If-None-Match") || '';
-    my $etag = '"'.md5_hex($data).'"';
+    my $etag = '"'.md5_hex(Encode::encode_utf8($data)).'"';
     $self->send_header('ETag', $etag);
     return $etag eq $req_tag;
 }
 
+sub check_last_modified {
+	my ($self, $last_modified) = @_;
+	return 0 if not $last_modified;
+
+	my $last_modified_str = HTTP::Date::time2str($last_modified);
+	$self->send_header('Last-Modified', $last_modified_str);
+
+	my $since = $self->get_header('If-Modified-Since') || return 0;
+	$since =~ s/;.+$//; # remove any parameters
+
+	return 1 if $since eq $last_modified_str; # optimization
+	my $since_epoch = HTTP::Date::str2time($since) || return 0;
+	return 1 if $since_epoch >= $last_modified;
+
+	return 0; # modified
+}
+
 =head2 report_error
 
     $self->report_error($title, $message)
 
 Used when something went horribly wrong inside Bryar. Spits out the
-error in as friendly a way as possible to the browser.
+error in as friendly a way as possible to the browser, HTML-escaped
+and enclosed by a <p> tag, and to STDERR.
 
 =cut
 
-sub report_error {
+sub report_error_browser {
     my ($class, $title, $message) = @_;
     $class->send_header("Content-type", "text/html");
-    $class->send_data("<H1>$title</H1> $message");
-    exit;
+    $class->send_header('Status', '500');
+    $class->send_data(
+        "<!DOCTYPE HTML>\n" .
+        "<html><head><title>$title</title></head>\n" .
+        "<body><h1>$title</h1>$message</body></html>"
+    );
+}
+
+sub report_error_html {
+    my ($class, $title, $message) = @_;
+    $class->report_error_browser($title, $message);
+    croak "$title: $message";
+}
+
+sub report_error {
+    my ($class, $title, $message) = @_;
+    my ($texttitle, $textmessage) = ($title, $message);
+    $title   =~ s/&/&amp;/g; $title   =~ s/</&lt;/g; $title   =~ s/>/&gt;/g;
+    $message =~ s/&/&amp;/g; $message =~ s/</&lt;/g; $message =~ s/>/&gt;/g;
+    $message = "<p>$message</p>";
+    $class->report_error_browser($title, $message);
+    croak "$texttitle: $textmessage";
 }
 
 sub init {
