@@ -23,7 +23,7 @@ sub obtain_url { $cgi->url() }
 sub obtain_path_info { $cgi->path_info() }
 sub obtain_params { map { $_ => $cgi->param($_) } $cgi->param }
 sub get_header { my $self = shift; $cgi->http(shift) }
-sub send_data { my $self = shift; print "\n",@_ }
+sub send_data { my $self = shift; binmode(STDOUT, ':utf8'); print "\n",@_ }
 sub send_header { my ($self, $k, $v) = @_; print "$k: $v\n"; }
 
 1;
